########################################################################################################
##						zCivics						      ##
##												      ##
##	Created By:		Zebra 9								      ##
##	Created:		5 / 21 / 07							      ##
##		Updated:	5 /  5 / 08							      ##
##			Added zConfig								      ##
##			Added Civic prereqs for techs						      ##
##			Added Civic prereqs for projects					      ##
##			Added ability to allow continuation of unit and building production without   ##
##				correct civics							      ##
########################################################################################################

from CvPythonExtensions import *
import zConfigParser
import os
import zConfig

# globals
gc = CyGlobalContext()

class zCivics:
	def __init__(self, manager):
		## Read zCNFG Here
		path = os.path.join("Assets", "zConfig", "zCivics.zcnfg")
		info = zConfigParser.parse(path, myZCnfgParser, log = True)


		self.civics = info.civics
		self.units = info.units
		self.buildings = info.buildings
		self.techs = info.techs
		self.projects = info.projects

		manager.addCallbackHandler("cannotDoCivic",	self.cannotDoCivic)
		manager.addCallbackHandler("cannotTrain",	self.cannotTrain)
		manager.addCallbackHandler("cannotConstruct",	self.cannotConstruct)
		manager.addCallbackHandler("cannotResearch",	self.cannotResearch)
		manager.addCallbackHandler("cannotCreate",	self.cannotCreate)

	# Logic Code Here
	def cannotDoCivic(self, args):
		"""See if we can adopt the civic"""
		iPlayer	= args[0]
		iCivic	= args[1]

		pCivic = self.civics.get(gc.getCivicInfo(iCivic).getType(), None)
		if pCivic:
			pPlayer = gc.getPlayer(iPlayer)

			## Make sure that the player's state religion permits this civic
			if pPlayer.getStateReligion() >= 0:
				szReligionType = gc.getReligionInfo(pPlayer.getStateReligion()).getType()
			else:
				szReligionType = "RELIGION_NO_State"

			## If the religion isn't listed return True so this player can not recieve this civic
			if pCivic.religionPrereq != [] and szReligionType not in pCivic.religionPrereq:
				return True



			## Make sure that the player's current civic isn't locked
			opt = gc.getCivicInfo(iCivic).getCivicOptionType()
			curCivic = self.civics.get(gc.getCivicInfo(pPlayer.getCivics(opt)).getType(), None)

			## if curCivic is a lockable civic return True so that the player will not be able to change
			if curCivic and curCivic.isLock:
				return True



			## Make sure that this civ can adopt this civic
			civ = gc.getCivilizationInfo(pPlayer.getCivilizationType()).getType()

			## if the civic is blocked return True so that the player can not adotp this civic
			if civ in pCivic.civBlock:
				return True



			## Make sure that this leader can adopt this civic
			leader = gc.getLeaderHeadInfo(pPlayer.getLeaderType()).getType()

			## if the civic is blocked return True so that the player can not adotp this civic
			if leader in pCivic.leaderBlock:
				return True



			## Make sure that the player has the correct civics
			if pCivic.civicPrereq != []:
				for i in range(gc.getNumCivicOptionInfos()):
					if gc.getCivicInfo(pPlayer.getCivics(i)).getType() not in pCivic.civicPrereq and i in pCivic.civicOptionPrereq:
						return True

		return False

	def cannotTrain(self, args):
		"""See if we can train the unit"""
		pCity		= args[0]
		iUnit		= args[1]
		bContinue	= args[2]
		bTestVisible	= args[3]
		bIgnoreCost	= args[4]
		bIgnoreUpgrades	= args[5]

		pUnit = self.units.get(gc.getUnitInfo(iUnit).getType(), None)
		if pUnit:
			pPlayer = gc.getPlayer(pCity.getOwner())
			if not bContinue or pUnit.bExtraValue:
				for i in range(gc.getNumCivicOptionInfos()):
					if gc.getCivicInfo(pPlayer.getCivics(i)).getType() not in pUnit.civicPrereq and i in pUnit.civicOptionPrereq:
						return True

		return False

	def cannotConstruct(self, args):
		"""See if we can construct the building"""
		pCity		= args[0]
		iBuilding	= args[1]
		bContinue	= args[2]
		bTestVisible	= args[3]
		bIgnoreCost	= args[4]

		pBuilding = self.buildings.get(gc.getBuildingInfo(iBuilding).getType(), None)
		if pBuilding:
			pPlayer = gc.getPlayer(pCity.getOwner())
			if not bContinue or pBuilding.bExtraValue:
				for i in range(gc.getNumCivicOptionInfos()):
					if gc.getCivicInfo(pPlayer.getCivics(i)).getType() not in pBuilding.civicPrereq and i in pBuilding.civicOptionPrereq:
						return True

		return False

	def cannotResearch(self, args):
		"""See if we can research the tech"""
		iPlayer	= args[0]
		iTech	= args[1]
		bTrade	= args[2]

		pTech = self.techs.get(gc.getTechInfo(iTech).getType(), None)
		if pTech:
			pPlayer = gc.getPlayer(iPlayer)
			if not bContinue or pTech.bExtraValue:
				for i in range(gc.getNumCivicOptionInfos()):
					if gc.getCivicInfo(pPlayer.getCivics(i)).getType() not in pTech.civicPrereq and i in pTech.civicOptionPrereq:
						return True

		return False

	def cannotCreate(self, args):
		"""See if we can create the project"""
		pCity		= args[0]
		iProject	= args[1]
		bContinue	= args[2]
		bTestVisible	= args[3]

		pProj = self.projects.get(gc.getProjectInfo(iProject).getType(), None)
		if pProj:
			pPlayer = gc.getPlayer(iPlayer)
			if not bContinue or pProj.bExtraValue:
				for i in range(gc.getNumCivicOptionInfos()):
					if gc.getCivicInfo(pPlayer.getCivics(i)).getType() not in pProj.civicPrereq and i in pProj.civicOptionPrereq:
						return True

		return False

class civicInfo:
	def __init__(self):
		self.civicPrereq = []
		self.civicOptionPrereq = []
		self.religionPrereq = []
		self.leaderBlock = []
		self.civBlock = []
		self.isLock = False

class itemInfo:
	def __init__(self):
		self.civicPrereq = []
		self.civicOptionPrereq = []
		self.bExtraValue = False

class myZCnfgParser(zConfig.parser):
	civics = dict()
	units = dict()
	buildings = dict()
	techs = dict()
	projects = dict()
	currentType = ""
	currentName = ""
	def createValue(self, node, value):
		name = node.getName().lower()
		if self.currentType == "civic":
			if name == "type":
				self.currentName = value
				self.civics[self.currentName] = civicInfo()
			elif name == "civic":
				self.civics[self.currentName].civicPrereq.append(value)
				self.civics[self.currentName].civicOptionPrereq.append(gc.getCivicInfo(gc.getInfoTypeForString(value)).getCivicOptionType())
			elif name == "religion":
				self.civics[self.currentName].religionPrereq.append(value)
			elif name == "leader":
				self.civics[self.currentName].leaderBlock.append(value)
			elif name == "civ":
				self.civics[self.currentName].civBlock.append(value)
			elif name == "lock":
				try:
					val = bool(value)
				except:
					val = False
				self.civics[self.currentName].lock = val

		elif self.currentType == "unit":
			if name == "type":
				self.currentName = value
				self.units[self.currentName] = itemInfo()
			elif name == "civic":
				self.units[self.currentName].civicPrereq.append(value)
				self.units[self.currentName].civicOptionPrereq.append(gc.getCivicInfo(gc.getInfoTypeForString(value)).getCivicOptionType())
			elif name == "allow continue":
				try:
					val = bool(value)
				except:
					val = False
				self.units[self.currentName].bExtraValue = val

		elif self.currentType == "building":
			if name == "type":
				self.currentName = value
				self.buildings[self.currentName] = itemInfo()
			elif name == "civic":
				self.buildings[self.currentName].civicPrereq.append(value)
				self.buildings[self.currentName].civicOptionPrereq.append(gc.getCivicInfo(gc.getInfoTypeForString(value)).getCivicOptionType())
			elif name == "allow continue":
				try:
					val = bool(value)
				except:
					val = False
				self.buildings[self.currentName].bExtraValue = val

		elif self.currentType == "tech":
			if name == "type":
				self.currentName = value
				self.techs[self.currentName] = itemInfo()
			elif name == "civic":
				self.techs[self.currentName].civicPrereq.append(value)
				self.techs[self.currentName].civicOptionPrereq.append(gc.getCivicInfo(gc.getInfoTypeForString(value)).getCivicOptionType())
			elif name == "allow trade":
				try:
					val = bool(value)
				except:
					val = False
				self.techs[self.currentName].bExtraValue = val

		elif self.currentType == "project":
			if name == "type":
				self.currentName = value
				self.projects[self.currentName] = itemInfo()
			elif name == "civic":
				self.projects[self.currentName].civicPrereq.append(value)
				self.projects[self.currentName].civicOptionPrereq.append(gc.getCivicInfo(gc.getInfoTypeForString(value)).getCivicOptionType())
			elif name == "allow continue":
				try:
					val = bool(value)
				except:
					val = False
				self.projects[self.currentName].bExtraValue = val

		return

	def createEndTag(self, node):
		name = node.getName().lower()
		if name in ["civic", "unit", "building", "tech", "project"] and self.currentType != "":
			self.currentType = ""
		return

	def createStartTag(self, node):
		name = node.getName().lower()
		if name in ["civic", "unit", "building", "tech", "project"] and self.currentType == "":
			self.currentType = name
			self.currentName = ""
		return

def GameUtils(manager):
	civics = zCivics(manager)